require 'net/http'
require 'rexml/document'
require 'time'

class Tweet
  attr_reader :text, :t_id, :reply_person, :reply_tweet, :time

  def initialize(e)
    @text = munge(child(e, 'text'))
    @t_id = child(e, 'id').to_i
    c = child(e, 'in_reply_to_screen_name')
    @reply_person = c == '' ? nil : c
    c = child(e, 'in_reply_to_status_id')
    @reply_tweet = c == '' ? nil : c
    @time = Time.parse(child(e, 'created_at'))
  end

  private

  def child(tweet, type)
    tweet.elements[type].get_text.to_s
  end

  def munge(text)
    if text =~ /^(.*\()(http:\/\/[^)]*)(\).*$)/m
      munge($1) + check_url($2) + munge($3)    
    elsif text =~ /^(.*)(http:\/\/\S+)(.*)$/m
      munge($1) + check_url($2) + munge($3)
    else
      text.gsub('&amp;gt;', '&gt;').gsub('&amp;lt;', '&lt;')
    end
  end

  def check_url(u)
    trailer = ''
    if (u =~ /([,\)])$/)
      trailer = $1
      u.chop!
    end
    begin
      uri = URI.parse(u)
      req = Net::HTTP::Head.new(u)
      http = Net::HTTP.new(uri.host, uri.port)
      http.start do |connection|
        resp = connection.request(req)
        if resp.class == Net::HTTPMovedPermanently
          u = resp.header['Location'].gsub('&', '&amp;')
        end
      end
    rescue => message
      puts "Blew up on #{u}: #{message}"
    end
    if u =~ %r{^http://(.*)$}
      label = $1
    end
    "<a href='#{u}'>#{label}</a>#{trailer}"
  end

end
